@@ -1,4 +1,4 @@
-/* $Id: wmesa.h,v 3.0 1998/02/20 05:06:59 brianp Exp $ */
+/* $Id: wmesa.h,v 1.1.1.1 1999/08/19 00:55:40 jtg Exp $ */
 
 /*
  * Mesa 3-D graphics library
@@ -24,6 +24,15 @@
 
 /*
  * $Log: wmesa.h,v $
+ * Revision 1.1.1.1  1999/08/19 00:55:40  jtg
+ * Imported sources
+ *
+ * Revision 3.2  1999/01/03 02:54:45  brianp
+ * updated per Ted Jump
+ *
+ * Revision 3.1  1998/12/01 02:34:27  brianp
+ * applied Mark Kilgard's patches from November 30, 1998
+ *
  * Revision 3.0  1998/02/20 05:06:59  brianp
  * initial rev
  *
@@ -60,7 +69,6 @@ extern "C" {
 #endif
 
 
-#include <windows.h>
 #include "gl\gl.h"
 
 #pragma warning (disable:4273)
@@ -109,6 +117,7 @@ extern WMesaContext WMesaCreateContext(HWND hWnd,HPALETTE* pPal,
 extern void WMesaDestroyContext( void );
 
 
+
 /*
  * Make the specified context the current one.
  */
