@@ -1,9 +1,10 @@
+/* $Id: fxdd.c,v 1.84 2001/09/23 16:50:01 brianp Exp $ */
 
 /*
  * Mesa 3-D graphics library
- * Version:  3.3
+ * Version:  4.0
  *
- * Copyright (C) 1999-2000  Brian Paul   All Rights Reserved.
+ * Copyright (C) 1999-2001  Brian Paul   All Rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -21,26 +22,14 @@
  * BRIAN PAUL BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN
  * AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
  * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
- *
- *
- * Original Mesa / 3Dfx device driver (C) 1999 David Bucciarelli, by the
- * terms stated above.
- *
- * Thank you for your contribution, David!
- *
- * Please make note of the above copyright/license statement.  If you
- * contributed code or bug fixes to this code under the previous (GNU
- * Library) license and object to the new license, your code will be
- * removed at your request.  Please see the Mesa docs/COPYRIGHT file
- * for more information.
- *
- * Additional Mesa/3Dfx driver developers:
- *   Daryll Strauss <daryll@precisioninsight.com>
- *   Keith Whitwell <keith@precisioninsight.com>
- *
- * See fxapi.h for more revision/author details.
  */
 
+/* Authors:
+ *    David Bucciarelli
+ *    Brian Paul
+ *    Daryll Strauss
+ *    Keith Whitwell
+ */
 
 /* fxdd.c - 3Dfx VooDoo Mesa device driver functions */
 
@@ -657,28 +646,16 @@ int
 fxDDInitFxMesaContext(fxMesaContext fxMesa)
 {
    int i;
-   static int firsttime = 1;
 
    for (i = 0; i < 256; i++) {
       gl_ubyte_to_float_255_color_tab[i] = (float) i;
    }
 
-   if (firsttime) {
-      fxDDSetupInit();
-      fxDDTrifuncInit();
-      firsttime = 0;
-   }
-
    FX_setupGrVertexLayout();
 
    if (getenv("FX_EMULATE_SINGLE_TMU"))
       fxMesa->haveTwoTMUs = GL_FALSE;
 
-   fxMesa->emulateTwoTMUs = fxMesa->haveTwoTMUs;
-
-   if (!getenv("FX_DONT_FAKE_MULTITEX"))
-      fxMesa->emulateTwoTMUs = GL_TRUE;
-
    if (getenv("FX_GLIDE_SWAPINTERVAL"))
       fxMesa->swapInterval = atoi(getenv("FX_GLIDE_SWAPINTERVAL"));
    else
@@ -754,7 +731,7 @@ fxDDInitFxMesaContext(fxMesaContext fxMesa)
 
    fxMesa->textureAlign = FX_grGetInteger(FX_TEXTURE_ALIGN);
    fxMesa->glCtx->Const.MaxTextureLevels = 9;
-   fxMesa->glCtx->Const.MaxTextureUnits = fxMesa->emulateTwoTMUs ? 2 : 1;
+   fxMesa->glCtx->Const.MaxTextureUnits = fxMesa->haveTwoTMUs ? 2 : 1;
    fxMesa->new_state = _NEW_ALL;
 
    /* Initialize the software rasterizer and helper modules.
@@ -770,6 +747,7 @@ fxDDInitFxMesaContext(fxMesaContext fxMesa)
    fxAllocVB(fxMesa->glCtx);
 
    fxSetupDDPointers(fxMesa->glCtx);
+   fxDDInitTriFuncs(fxMesa->glCtx);
 
    /* Tell the software rasterizer to use pixel fog always.
     */
@@ -783,10 +761,6 @@ fxDDInitFxMesaContext(fxMesaContext fxMesa)
 
    fxDDInitExtensions(fxMesa->glCtx);
 
-#ifdef FXVTXFMT
-   fxDDInitVtxfmt(fxMesa->glCtx);
-#endif
-
    FX_grGlideGetState((GrState *) fxMesa->state);
 
    /* Run the config file */
@@ -830,7 +804,7 @@ fxDDInitExtensions(GLcontext * ctx)
    if (fxMesa->haveTwoTMUs)
       _mesa_enable_extension(ctx, "GL_EXT_texture_env_add");
 
-   if (fxMesa->emulateTwoTMUs)
+   if (fxMesa->haveTwoTMUs)
       _mesa_enable_extension(ctx, "GL_ARB_multitexture");
 }
 
@@ -843,8 +817,8 @@ fxDDInitExtensions(GLcontext * ctx)
  *
  * Performs similar work to fxDDChooseRenderState() - should be merged.
  */
-static GLboolean
-fxIsInHardware(GLcontext * ctx)
+GLboolean
+fx_check_IsInHardware(GLcontext * ctx)
 {
    fxMesaContext fxMesa = (fxMesaContext) ctx->DriverCtx;
 
@@ -868,7 +842,7 @@ fxIsInHardware(GLcontext * ctx)
    }
    /* Unsupported texture/multitexture cases */
 
-   if (fxMesa->emulateTwoTMUs) {
+   if (fxMesa->haveTwoTMUs) {
       if (ctx->Texture._ReallyEnabled & (TEXTURE0_3D | TEXTURE1_3D))
 	 return GL_FALSE;	/* can't do 3D textures */
       if (ctx->Texture._ReallyEnabled & (TEXTURE0_1D | TEXTURE1_1D))
@@ -939,6 +913,8 @@ fxIsInHardware(GLcontext * ctx)
    return GL_TRUE;
 }
 
+
+
 static void
 update_texture_scales(GLcontext * ctx)
 {
@@ -964,7 +940,7 @@ update_texture_scales(GLcontext * ctx)
 static void
 fxDDUpdateDDPointers(GLcontext * ctx, GLuint new_state)
 {
-   TNLcontext *tnl = TNL_CONTEXT(ctx);
+   /*   TNLcontext *tnl = TNL_CONTEXT(ctx);*/
    fxMesaContext fxMesa = FX_CONTEXT(ctx);
 
    _swrast_InvalidateState(ctx, new_state);
@@ -980,72 +956,29 @@ fxDDUpdateDDPointers(GLcontext * ctx, GLuint new_state)
 
    if (new_state & (_FX_NEW_IS_IN_HARDWARE |
 		    _FX_NEW_RENDERSTATE |
-		    _FX_NEW_SETUP_FUNCTION | _NEW_TEXTURE)) {
+		    _FX_NEW_SETUP_FUNCTION | 
+		    _NEW_TEXTURE)) {
 
       if (new_state & _FX_NEW_IS_IN_HARDWARE)
-	 fxMesa->is_in_hardware = fxIsInHardware(ctx);
+	 fxCheckIsInHardware(ctx);
 
       if (fxMesa->new_state)
 	 fxSetupFXUnits(ctx);
 
-      if (new_state & _FX_NEW_RENDERSTATE)
-	 fxDDChooseRenderState(ctx);
+      if (fxMesa->is_in_hardware) {
+	 if (new_state & _FX_NEW_RENDERSTATE)
+	    fxDDChooseRenderState(ctx);
 
-      if (new_state & _FX_NEW_SETUP_FUNCTION)
-	 tnl->Driver.BuildProjectedVertices = fx_validate_BuildProjVerts;
+	 if (new_state & _FX_NEW_SETUP_FUNCTION)
+	    fxChooseVertexState(ctx);
+      }
 
       if (new_state & _NEW_TEXTURE)
 	 update_texture_scales(ctx);
-
-   }
-
-#ifdef FXVTXFMT
-   if (fxMesa->allow_vfmt) {
-      if (new_state & _NEW_LIGHT)
-	 fx_update_lighting(ctx);
-
-      if (new_state & _FX_NEW_VTXFMT)
-	 fxDDCheckVtxfmt(ctx);
-   }
-#endif
-}
-
-static void
-fxDDRenderPrimitive(GLcontext * ctx, GLenum mode)
-{
-   fxMesaContext fxMesa = (fxMesaContext) ctx->DriverCtx;
-
-   if (!fxMesa->is_in_hardware) {
-      _swsetup_RenderPrimitive(ctx, mode);
-   }
-   else {
-      fxMesa->render_prim = mode;
    }
 }
 
 
-static void
-fxDDRenderStart(GLcontext * ctx)
-{
-   fxMesaContext fxMesa = (fxMesaContext) ctx->DriverCtx;
-
-   _swsetup_RenderStart(ctx);
-
-   if (fxMesa->new_state) {
-      fxSetupFXUnits(ctx);
-   }
-}
-
-static void
-fxDDRenderFinish(GLcontext * ctx)
-{
-   fxMesaContext fxMesa = (fxMesaContext) ctx->DriverCtx;
-
-   if (!fxMesa->is_in_hardware) {
-      _swsetup_RenderFinish(ctx);
-   }
-}
-
 
 
 void
@@ -1107,14 +1040,6 @@ fxSetupDDPointers(GLcontext * ctx)
    ctx->Driver.Enable = fxDDEnable;
 
    tnl->Driver.RunPipeline = _tnl_run_pipeline;
-   tnl->Driver.RenderStart = fxDDRenderStart;
-   tnl->Driver.RenderFinish = fxDDRenderFinish;
-   tnl->Driver.ResetLineStipple = _swrast_ResetLineStipple;
-   tnl->Driver.RenderPrimitive = fxDDRenderPrimitive;
-   tnl->Driver.RenderInterp = _swsetup_RenderInterp;
-   tnl->Driver.RenderCopyPV = _swsetup_RenderCopyPV;
-   tnl->Driver.RenderClippedLine = _swsetup_RenderClippedLine;
-   tnl->Driver.RenderClippedPolygon = _swsetup_RenderClippedPolygon;
 
    fxSetupDDSpanPointers(ctx);
    fxDDUpdateDDPointers(ctx, ~0);
