@@ -1,4 +1,4 @@
-/* $Id: xm_api.c,v 1.25 2001/06/15 14:18:46 brianp Exp $ */
+/* $Id: xm_api.c,v 1.29.2.1 2001/11/06 16:00:58 brianp Exp $ */
 
 /*
  * Mesa 3-D graphics library
@@ -1030,11 +1030,10 @@ static void setup_8bit_hpcr( XMesaVisual v )
  * Setup RGB rendering for a window with a True/DirectColor visual.
  */
 static void setup_truecolor( XMesaVisual v, XMesaBuffer buffer,
-                             XMesaWindow window, XMesaColormap cmap )
+                             XMesaColormap cmap )
 {
    unsigned long rmask, gmask, bmask;
    (void) buffer;
-   (void) window;
    (void) cmap;
 
    /* Compute red multiplier (mask) and bit shift */
@@ -1227,7 +1226,7 @@ static GLboolean initialize_visual_and_buffer( int client,
       int xclass;
       xclass = GET_VISUAL_CLASS(v);
       if (xclass==TrueColor || xclass==DirectColor) {
-	 setup_truecolor( v, b, (XMesaWindow)window, cmap );
+	 setup_truecolor( v, b, cmap );
       }
       else if (xclass==StaticGray && GET_VISUAL_DEPTH(v)==1) {
 	 setup_monochrome( v, b );
@@ -1261,7 +1260,7 @@ static GLboolean initialize_visual_and_buffer( int client,
     * reports bugs.
     */
    if (getenv("MESA_INFO")) {
-      fprintf(stderr, "X/Mesa visual = %p\n", v);
+      fprintf(stderr, "X/Mesa visual = %p\n", (void *) v);
       fprintf(stderr, "X/Mesa dithered pf = %u\n", v->dithered_pf);
       fprintf(stderr, "X/Mesa undithered pf = %u\n", v->undithered_pf);
       fprintf(stderr, "X/Mesa level = %d\n", v->level);
@@ -1636,10 +1635,7 @@ XMesaContext XMesaCreateContext( XMesaVisual v, XMesaContext share_list )
    }
 
    _mesa_enable_sw_extensions(ctx);
-   _mesa_enable_extension(ctx, "GL_ARB_texture_compression");
-   ctx->Driver.BaseCompressedTexFormat = _mesa_base_compressed_texformat;
-   ctx->Driver.CompressedTextureSize = _mesa_compressed_texture_size;
-   ctx->Driver.GetCompressedTexImage = _mesa_get_compressed_teximage;
+   _mesa_enable_1_3_extensions(ctx);
 
    if (CHECK_BYTE_ORDER(v)) {
       c->swapbytes = GL_FALSE;
@@ -1961,6 +1957,57 @@ XMesaBuffer XMesaCreatePixmapBuffer( XMesaVisual v,
 
 
 
+#if 0 /* not done */
+XMesaBuffer XMesaCreatePBuffer( XMesaVisual v, XMesaColormap cmap,
+                                unsigned int width, unsigned int height )
+{
+   int client = 0;
+   XMesaBuffer b = alloc_xmesa_buffer();
+   if (!b) {
+      return NULL;
+   }
+
+   b->xm_context = NULL; /* Associate no context with this buffer */
+
+   b->xm_visual = v;
+   b->pbuffer_flag = GL_TRUE;
+   b->display = v->display;
+   b->cmap = cmap;
+
+   /* determine back buffer implementation */
+   if (v->mesa_visual.doubleBufferMode) {
+      if (v->ximage_flag) {
+	 b->db_state = BACK_XIMAGE;
+      }
+      else {
+	 b->db_state = BACK_PIXMAP;
+      }
+   }
+   else {
+      b->db_state = 0;
+   }
+
+   _mesa_initialize_framebuffer(&b->mesa_buffer,
+                                &v->mesa_visual,
+                                v->mesa_visual.depthBits > 0,
+                                v->mesa_visual.stencilBits > 0,
+                                v->mesa_visual.accumRedBits +
+                                v->mesa_visual.accumGreenBits +
+                                v->mesa_visual.accumBlueBits > 0,
+                                v->mesa_visual.alphaBits > 0 );
+
+   if (!initialize_visual_and_buffer(client, v, b, v->mesa_visual.rgbMode,
+				     0, cmap)) {
+      free_xmesa_buffer(client, b);
+      return NULL;
+   }
+
+   return b;
+}
+#endif
+
+
+
 /*
  * Deallocate an XMesaBuffer structure and all related info.
  */
@@ -2032,6 +2079,10 @@ GLboolean XMesaMakeCurrent2( XMesaContext c, XMesaBuffer drawBuffer,
       if (drawBuffer->FXctx) {
          fxMesaMakeCurrent(drawBuffer->FXctx);
 
+         /* Disassociate drawBuffer's current context from drawBuffer */
+         if (drawBuffer->xm_context)
+            drawBuffer->xm_context->xm_buffer = NULL;
+
          /* Disassociate old buffer from this context */
          if (c->xm_buffer)
             c->xm_buffer->xm_context = NULL;
@@ -2054,9 +2105,14 @@ GLboolean XMesaMakeCurrent2( XMesaContext c, XMesaBuffer drawBuffer,
          return GL_TRUE;
       }
 
+      /* Disassociate drawBuffer's current context from drawBuffer */
+      if (drawBuffer->xm_context)
+         drawBuffer->xm_context->xm_buffer = NULL;
+ 
       /* Disassociate old buffer with this context */
       if (c->xm_buffer)
 	  c->xm_buffer->xm_context = NULL;
+
       drawBuffer->xm_context = c; /* Associate the context with this buffer */
 
       c->xm_buffer = drawBuffer;
@@ -2506,7 +2562,7 @@ const char *XMesaGetString( XMesaContext c, int name )
 {
    (void) c;
    if (name==XMESA_VERSION) {
-      return "3.1";
+      return "3.5";
    }
    else if (name==XMESA_EXTENSIONS) {
       return "";
