@@ -1,4 +1,4 @@
-/* $Id: attrib.c,v 1.52 2001/06/18 17:26:08 brianp Exp $ */
+/* $Id: attrib.c,v 1.57 2001/09/18 15:27:18 kschultz Exp $ */
 
 /*
  * Mesa 3-D graphics library
@@ -49,6 +49,7 @@
 #include "polygon.h"
 #include "simple_list.h"
 #include "stencil.h"
+#include "texobj.h"
 #include "texstate.h"
 #include "mtypes.h"
 #endif
@@ -79,8 +80,8 @@ static void
 copy_texobj_state( struct gl_texture_object *dest,
                    const struct gl_texture_object *src )
 {
-   /*
    dest->Name = src->Name;
+   /*
    dest->Dimensions = src->Dimensions;
    */
    dest->Priority = src->Priority;
@@ -219,6 +220,7 @@ _mesa_PushAttrib(GLbitfield mask)
       attr->Map2Vertex3 = ctx->Eval.Map2Vertex3;
       attr->Map2Vertex4 = ctx->Eval.Map2Vertex4;
       attr->Normalize = ctx->Transform.Normalize;
+      attr->RasterPositionUnclipped = ctx->Transform.RasterPositionUnclipped;
       attr->PixelTexture = ctx->Pixel.PixelTextureEnabled;
       attr->PointSmooth = ctx->Point.SmoothFlag;
       attr->PolygonOffsetPoint = ctx->Polygon.OffsetPoint;
@@ -440,7 +442,6 @@ pop_enable_group(GLcontext *ctx, const struct gl_enable_attrib *enable)
 	}
 
    TEST_AND_UPDATE(ctx->Color.AlphaEnabled, enable->AlphaTest, GL_ALPHA_TEST);
-   TEST_AND_UPDATE(ctx->Transform.Normalize, enable->AutoNormal, GL_NORMALIZE);
    TEST_AND_UPDATE(ctx->Color.BlendEnabled, enable->Blend, GL_BLEND);
 
    for (i=0;i<MAX_CLIP_PLANES;i++) {
@@ -499,9 +500,13 @@ pop_enable_group(GLcontext *ctx, const struct gl_enable_attrib *enable)
                    GL_MAP2_VERTEX_3);
    TEST_AND_UPDATE(ctx->Eval.Map2Vertex4, enable->Map2Vertex4,
                    GL_MAP2_VERTEX_4);
+   TEST_AND_UPDATE(ctx->Eval.AutoNormal, enable->AutoNormal, GL_AUTO_NORMAL);
    TEST_AND_UPDATE(ctx->Transform.Normalize, enable->Normalize, GL_NORMALIZE);
    TEST_AND_UPDATE(ctx->Transform.RescaleNormals, enable->RescaleNormals,
                    GL_RESCALE_NORMAL_EXT);
+   TEST_AND_UPDATE(ctx->Transform.RasterPositionUnclipped,
+                   enable->RasterPositionUnclipped,
+                   GL_RASTER_POSITION_UNCLIPPED_IBM);
    TEST_AND_UPDATE(ctx->Pixel.PixelTextureEnabled, enable->PixelTexture,
                    GL_POINT_SMOOTH);
    TEST_AND_UPDATE(ctx->Point.SmoothFlag, enable->PointSmooth,
@@ -588,27 +593,20 @@ pop_texture_group(GLcontext *ctx, const struct gl_texture_attrib *texAttrib)
 {
    GLuint u;
 
-   /* "un-bump" the texture object reference counts.  We did that so they
-    * wouldn't inadvertantly get deleted.
-    */
-   for (u = 0; u < ctx->Const.MaxTextureUnits; u++) {
-      ctx->Texture.Unit[u].Current1D->RefCount--;
-      ctx->Texture.Unit[u].Current2D->RefCount--;
-      ctx->Texture.Unit[u].Current3D->RefCount--;
-      ctx->Texture.Unit[u].CurrentCubeMap->RefCount--;
-   }
-
    for (u = 0; u < ctx->Const.MaxTextureUnits; u++) {
       const struct gl_texture_unit *unit = &texAttrib->Unit[u];
       GLuint numObjs, i;
 
       _mesa_ActiveTextureARB(GL_TEXTURE0_ARB + u);
-      _mesa_set_enable(ctx, GL_TEXTURE_1D, unit->Enabled & TEXTURE0_1D);
-      _mesa_set_enable(ctx, GL_TEXTURE_2D, unit->Enabled & TEXTURE0_2D);
-      _mesa_set_enable(ctx, GL_TEXTURE_3D, unit->Enabled & TEXTURE0_3D);
+      _mesa_set_enable(ctx, GL_TEXTURE_1D,
+              (GLboolean) (unit->Enabled & TEXTURE0_1D ? GL_TRUE : GL_FALSE));
+      _mesa_set_enable(ctx, GL_TEXTURE_2D,
+              (GLboolean) (unit->Enabled & TEXTURE0_2D ? GL_TRUE : GL_FALSE));
+      _mesa_set_enable(ctx, GL_TEXTURE_3D,
+              (GLboolean) (unit->Enabled & TEXTURE0_3D ? GL_TRUE : GL_FALSE));
       if (ctx->Extensions.ARB_texture_cube_map) {
          _mesa_set_enable(ctx, GL_TEXTURE_CUBE_MAP_ARB,
-                          unit->Enabled & TEXTURE0_CUBE);
+             (GLboolean) (unit->Enabled & TEXTURE0_CUBE ? GL_TRUE : GL_FALSE));
       }
       _mesa_TexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, unit->EnvMode);
       _mesa_TexEnvfv(GL_TEXTURE_ENV, GL_TEXTURE_ENV_COLOR, unit->EnvColor);
@@ -693,6 +691,8 @@ pop_texture_group(GLcontext *ctx, const struct gl_texture_attrib *texAttrib)
             ; /* silence warnings */
          }
 
+         _mesa_BindTexture(target, obj->Name);
+
          bordColor[0] = CHAN_TO_FLOAT(obj->BorderColor[0]);
          bordColor[1] = CHAN_TO_FLOAT(obj->BorderColor[1]);
          bordColor[2] = CHAN_TO_FLOAT(obj->BorderColor[2]);
@@ -728,6 +728,17 @@ pop_texture_group(GLcontext *ctx, const struct gl_texture_attrib *texAttrib)
    }
    _mesa_ActiveTextureARB(GL_TEXTURE0_ARB
                           + texAttrib->CurrentUnit);
+
+   /* "un-bump" the texture object reference counts.  We did that so they
+    * wouldn't inadvertantly get deleted while they were still referenced
+    * inside the attribute state stack.
+    */
+   for (u = 0; u < ctx->Const.MaxTextureUnits; u++) {
+      ctx->Texture.Unit[u].Current1D->RefCount--;
+      ctx->Texture.Unit[u].Current2D->RefCount--;
+      ctx->Texture.Unit[u].Current3D->RefCount--;
+      ctx->Texture.Unit[u].CurrentCubeMap->RefCount--;
+   }
 }
 
 
@@ -778,7 +789,7 @@ _mesa_PopAttrib(void)
             {
                const struct gl_colorbuffer_attrib *color;
                color = (const struct gl_colorbuffer_attrib *) attr->data;
-               _mesa_ClearIndex(color->ClearIndex);
+               _mesa_ClearIndex((GLfloat) color->ClearIndex);
                _mesa_ClearColor(CHAN_TO_FLOAT(color->ClearColor[0]),
                                 CHAN_TO_FLOAT(color->ClearColor[1]),
                                 CHAN_TO_FLOAT(color->ClearColor[2]),
@@ -1123,17 +1134,17 @@ _mesa_PopClientAttrib(void)
          case GL_CLIENT_PACK_BIT:
             MEMCPY( &ctx->Pack, attr->data,
                     sizeof(struct gl_pixelstore_attrib) );
-	    ctx->NewState = _NEW_PACKUNPACK;
+	    ctx->NewState |= _NEW_PACKUNPACK;
             break;
          case GL_CLIENT_UNPACK_BIT:
             MEMCPY( &ctx->Unpack, attr->data,
                     sizeof(struct gl_pixelstore_attrib) );
-	    ctx->NewState = _NEW_PACKUNPACK;
+	    ctx->NewState |= _NEW_PACKUNPACK;
             break;
          case GL_CLIENT_VERTEX_ARRAY_BIT:
             MEMCPY( &ctx->Array, attr->data,
 		    sizeof(struct gl_array_attrib) );
-	    ctx->NewState = _NEW_ARRAY;
+	    ctx->NewState |= _NEW_ARRAY;
             break;
          default:
             _mesa_problem( ctx, "Bad attrib flag in PopClientAttrib");
