@@ -1,21 +1,21 @@
-/* $Id: config.c,v 1.10.4.1 2000/10/17 00:24:11 brianp Exp $ */
+/* $Id: config.c,v 1.16 2001/03/12 00:48:37 gareth Exp $ */
 
 /*
  * Mesa 3-D graphics library
- * Version:  3.4
- * 
- * Copyright (C) 1999-2000  Brian Paul   All Rights Reserved.
- * 
+ * Version:  3.5
+ *
+ * Copyright (C) 1999-2001  Brian Paul   All Rights Reserved.
+ *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
  * to deal in the Software without restriction, including without limitation
  * the rights to use, copy, modify, merge, publish, distribute, sublicense,
  * and/or sell copies of the Software, and to permit persons to whom the
  * Software is furnished to do so, subject to the following conditions:
- * 
+ *
  * The above copyright notice and this permission notice shall be included
  * in all copies or substantial portions of the Software.
- * 
+ *
  * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
  * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
@@ -29,7 +29,7 @@
  * Copyright (C) 1999 Keith Whitwell.
  *
  * I hate parsers, so I've choosen a lisp-like syntax - extremely easy
- * to parse and potentially very expressive.  
+ * to parse and potentially very expressive.
  */
 
 
@@ -37,13 +37,13 @@
 #include "all.h"
 #else
 #include "glheader.h"
-#include "config.h"
+#include "context.h"
 #include "enums.h"
 #include "extensions.h"
 #include "hint.h"
 #include "simple_list.h"
 #include "mem.h"
-#include "types.h"
+#include "mtypes.h"
 #endif
 
 
@@ -62,7 +62,7 @@ struct cnode {
  */
 static void pad(int n) { putchar('\n'); while(n--) putchar(' '); }
 
-static void print_list( struct cnode *n, int indent ) 
+static void print_list( struct cnode *n, int indent )
 {
    int i = 0;
    printf("( ");
@@ -90,7 +90,7 @@ static void print_list( struct cnode *n, int indent )
 
 /* Accessors to query the contents of a cnode.
  */
-static int is_list( struct cnode *x, struct cnode **h, struct cnode **t) 
+static int is_list( struct cnode *x, struct cnode **h, struct cnode **t)
 {
    if (x->type == list_t) {
       struct cnode *tmp = x;
@@ -101,28 +101,28 @@ static int is_list( struct cnode *x, struct cnode **h, struct cnode **t)
    return 0;
 }
 
-static int is_nil( const struct cnode *x ) 
+static int is_nil( const struct cnode *x )
 {
    return x->type == nil_t;
 }
 
-static int is_word( struct cnode *x, const char **s ) 
+static int is_word( struct cnode *x, const char **s )
 {
    if (x->type == word_t) {
       *s = x->data.w.text;
       return 1;
-   } 
+   }
    return 0;
 }
 
-static int match_word( struct cnode *x, const char *s ) 
+static int match_word( struct cnode *x, const char *s )
 {
-   if (x->type == word_t) 
+   if (x->type == word_t)
       return strcmp(s, x->data.w.text) == 0;
    return 0;
 }
 
-/* Build the parsed expression. 
+/* Build the parsed expression.
  */
 static void skip_comment( FILE *file )
 {
@@ -136,10 +136,10 @@ static struct cnode *get_word( int line, FILE *file )
 {
    int sz = 16, len = 0;
    char *text = (char *) MALLOC( sz * sizeof(char) );
-   
+
    while (1) {
       int c = getc(file);
-      if (len == sz)  
+      if (len == sz)
 	 text = (char *) realloc( text, sizeof(char) * (sz *= 2) );
       if (c == EOF || isspace(c) || c == ')') {
 	 struct cnode *n = MALLOC_STRUCT(cnode);
@@ -149,7 +149,7 @@ static struct cnode *get_word( int line, FILE *file )
 	 n->line = line;
 	 n->data.w.text = text;
 	 return n;
-      }   
+      }
       else
 	 text[len++] = c;
    }
@@ -167,19 +167,19 @@ static struct cnode *get_list( int *line, FILE *file )
       int c = getc(file);
 
       switch (c) {
-      case EOF: return head; 
+      case EOF: return head;
       case ')': return head;
       case ';': skip_comment( file ); continue;
       case '\n': (*line)++; continue;
-      case '(': 
+      case '(':
 	 n = get_list( line, file );
 	 break;
-      default: 
+      default:
 	 if (isspace(c)) continue;
-	 ungetc(c, file); 
+	 ungetc(c, file);
 	 n = get_word( *line, file );
 	 break;
-      } 
+      }
 
       (*current)->type = list_t;
       (*current)->data.l.head = n;
@@ -202,15 +202,14 @@ static void disable_extension( GLcontext *ctx, struct cnode *args )
    struct cnode *head, *tail;
    const char *c;
 
-   if (is_list(args, &head, &tail) && 
+   if (is_list(args, &head, &tail) &&
        is_nil(tail) &&
-       is_word(head, &c)) 
-   {
-      if (gl_extensions_disable( ctx, c ) != 0)
-	 error( head, "unknown extension" );
+       is_word(head, &c)) {
+      _mesa_disable_extension( ctx, c );
+   }
+   else {
+      error( args, "bad args for disable-extension" );
    }
-   else
-      error( args, "bad args for disable-extension" );	    
 }
 
 
@@ -219,30 +218,28 @@ static void default_hint( GLcontext *ctx, struct cnode *args )
    struct cnode *hint, *tail, *value;
    const char *hname, *vname;
 
-   if (is_list(args, &hint, &tail) && 
+   if (is_list(args, &hint, &tail) &&
        is_list(tail, &value, &tail) &&
        is_nil(tail) &&
        is_word(hint, &hname) &&
        is_word(value, &vname))
    {
-      GLint h = gl_lookup_enum_by_name(hname);
-      GLint v = gl_lookup_enum_by_name(vname);
+      GLint h = _mesa_lookup_enum_by_name(hname);
+      GLint v = _mesa_lookup_enum_by_name(vname);
       if (h != -1 && v != -1)
       {
-	 printf("calling glHint(%s=%d, %s=%d)\n", hname, h, vname, v);
 	 if (!_mesa_try_Hint( ctx, (GLenum) h, (GLenum) v ))
 	    error( hint, "glHint failed");
-	 printf("allow draw mem: %d\n", ctx->Hint.AllowDrawMem);
 	 return;
       }
       else
 	 error( hint, "unknown or illegal value for default-hint" );
    }
    else
-      error( args, "bad args for default-hint" );	    
+      error( args, "bad args for default-hint" );
 }
 
-/* Use the general-purpose set-variable 
+/* Use the general-purpose set-variable
  */
 static void fx_catch_signals( GLcontext *ctx, struct cnode *args )
 {
@@ -253,7 +250,7 @@ static void fx_catch_signals( GLcontext *ctx, struct cnode *args )
 /*  	  "fx-catch-signals deprecated, use " */
 /*  	  "(set-variable fx-catch-signals ...) instead"); */
 
-   if (is_list(args, &head, &tail) && 
+   if (is_list(args, &head, &tail) &&
        is_nil(tail) &&
        is_word(head, &value)) {
       if (strcmp(value, "false") == 0)
@@ -286,11 +283,11 @@ static void set_var( GLcontext *ctx, struct cnode *args )
    struct cnode *head, *tail;
    const char *variable, *value;
 
-   if (is_list(args, &head, &tail) && 
+   if (is_list(args, &head, &tail) &&
        is_word(head, &variable) &&
        is_list(tail, &head, &tail) &&
        is_word(head, &value) &&
-       is_nil(tail)) 
+       is_nil(tail))
    {
       foreach(v, &varlist) {
 	 if (strcmp(v->name, variable) == 0) {
@@ -298,7 +295,7 @@ static void set_var( GLcontext *ctx, struct cnode *args )
 	    return;
 	 }
       }
-      
+
       error( head, "unknown variable" );
    }
    else {
@@ -306,8 +303,9 @@ static void set_var( GLcontext *ctx, struct cnode *args )
    }
 }
 
-void gl_register_config_var(const char *name, 
-			    void (*notify)( const char *, int ))
+void
+_mesa_register_config_var(const char *name,
+                          void (*notify)( const char *, int ))
 {
    struct var *v = MALLOC_STRUCT(var);
    v->name = name;
@@ -324,15 +322,15 @@ static void do_init( GLcontext *ctx, struct cnode *list )
       list = head;
       while (is_list(list, &head, &list)) {
 	 if (is_list(head, &func, &args)) {
-	    if (match_word(func, "disable-extension")) 
+	    if (match_word(func, "disable-extension"))
 	       disable_extension( ctx, args );
-	    else if (match_word(func, "default-hint")) 
+	    else if (match_word(func, "default-hint"))
 	       default_hint( ctx, args );
-	    else if (match_word(func, "fx-catch-signals")) 
+	    else if (match_word(func, "fx-catch-signals"))
 	       fx_catch_signals( ctx, args );
             else if (match_word(func, "set-variable"))
                set_var( ctx, args );
-	    else 
+	    else
 	       error( func, "unknown configuration method" );
          }
       }
@@ -349,7 +347,7 @@ static int run_init( GLcontext *ctx, const char *version, struct cnode *list )
 
    /* Uses the first matching init list.
     */
-   while (is_list(list, &head, &list)) 
+   while (is_list(list, &head, &list))
       if (is_list(head, &arg1, &head) &&
 	  is_list(head, &arg2, &head) &&
 	  match_word(arg1, "config-mesa") &&
@@ -359,7 +357,7 @@ static int run_init( GLcontext *ctx, const char *version, struct cnode *list )
 	    do_init( ctx, head );
 	    return 1;
 	 }
-      } 
+      }
       else
 	 error( head, "malformed toplevel configuration" );
 
@@ -368,21 +366,21 @@ static int run_init( GLcontext *ctx, const char *version, struct cnode *list )
 
 
 
-static void free_list( struct cnode *n ) 
+static void free_list( struct cnode *n )
 {
    while (n->type == list_t) {
-      struct cnode *tmp = n;      
+      struct cnode *tmp = n;
       switch (n->data.l.head->type) {
       case list_t:
 	 free_list( n->data.l.head );
 	 break;
       case word_t:
-	 FREE( n->data.l.head->data.w.text ); 
+	 FREE( n->data.l.head->data.w.text );
 	 FREE( n->data.l.head );
 	 break;
       case nil_t:
 	 FREE( n->data.l.head );
-	 break;	
+	 break;
       default:
 	 return;
       }
@@ -398,17 +396,17 @@ static void free_list( struct cnode *n )
 /* How paranoid do you have to be when reading a config file?  I don't
  * know half the ways to exploit this stuff, and given that this may
  * be run with root access, I think we're better off hardcoding the
- * pathname.  Some clever joe can fix this later if they care.  
+ * pathname.  Some clever joe can fix this later if they care.
  */
-void gl_read_config_file( GLcontext *ctx )
+void _mesa_read_config_file( GLcontext *ctx )
 {
    const char *default_config = "mesa3.1beta1";
 
-#if (defined(__WIN32__) && !defined(__CYGWIN__)) || defined(__MSDOS__)
+#if defined(__WIN32__) || defined(__MSDOS__)
    const char *filename = "mesa.cnf";
 #else
-   const char *filename = "/etc/mesa.conf"; 
-#endif   
+   const char *filename = "/etc/mesa.conf";
+#endif
    FILE *file;
    struct cnode *list;
    int line = 1;
@@ -418,12 +416,12 @@ void gl_read_config_file( GLcontext *ctx )
    int f;
    struct stat statbuf;
 
-   if ((f = open(filename, O_RDONLY)) == -1) 
+   if ((f = open(filename, O_RDONLY)) == -1)
       return;
 
-   if (fstat( f, &statbuf ) == -1 || 
+   if (fstat( f, &statbuf ) == -1 ||
        !S_ISREG( statbuf.st_mode ) ||
-       (file = fdopen(f, "r")) == 0) 
+       (file = fdopen(f, "r")) == 0)
    {
       close( f );
       return;
@@ -435,7 +433,7 @@ void gl_read_config_file( GLcontext *ctx )
 
    list = get_list( &line, file );
    fclose( file );
-   
+
    if ((v = getenv("MESA_CONFIG")) != 0 && *v != 0) {
       if (run_init( ctx, v, list )) {
 	 free_list( list );
@@ -444,16 +442,14 @@ void gl_read_config_file( GLcontext *ctx )
       else
 	 fprintf(stderr, "No configuration '%s' in init file\n", v);
    }
-	 
+
 
    if (!run_init( ctx, default_config, list )) {
       if (getenv("MESA_DEBUG")) {
-         fprintf(stderr, "No default configuration '%s' in init file\n", 
+         fprintf(stderr, "No default configuration '%s' in init file\n",
                  default_config);
       }
    }
 
    free_list( list );
 }
-
-
