@@ -1,4 +1,4 @@
-/* $Id: glheader.h,v 1.21 2001/06/15 15:22:07 brianp Exp $ */
+/* $Id: glheader.h,v 1.23 2001/10/17 13:31:07 brianp Exp $ */
 
 /*
  * Mesa 3-D graphics library
@@ -173,8 +173,12 @@ typedef struct tagPIXELFORMATDESCRIPTOR PIXELFORMATDESCRIPTOR, *PPIXELFORMATDESC
 
 
 #ifndef CAPI
+#ifdef WIN32
+#define CAPI _cdecl
+#else
 #define CAPI
 #endif
+#endif
 #include <GL/internal/glcore.h>
 
 
@@ -221,6 +225,10 @@ typedef struct tagPIXELFORMATDESCRIPTOR PIXELFORMATDESCRIPTOR, *PPIXELFORMATDESC
 #  define INLINE __inline__
 #elif defined(__MSC__)
 #  define INLINE __inline
+#elif defined(_MSC_VER)
+#  define INLINE __inline
+#elif defined(__ICL)
+#  define INLINE __inline
 #else
 #  define INLINE
 #endif
@@ -252,4 +260,12 @@ typedef struct tagPIXELFORMATDESCRIPTOR PIXELFORMATDESCRIPTOR, *PPIXELFORMATDESC
 typedef union { GLfloat f; GLint i; } fi_type;
 
 
+#ifndef GL_MIRRORED_REPEAT_ARB
+#define GL_MIRRORED_REPEAT_ARB  0x8370
+#endif
+#ifndef GL_ARB_texture_mirrored_repeat
+#define GL_ARB_texture_mirrored_repeat 1
+#endif
+
+
 #endif /* GLHEADER_H */
