@@ -1,4 +1,4 @@
-/* $Id: light.c,v 1.44 2001/05/09 12:24:51 keithw Exp $ */
+/* $Id: light.c,v 1.46 2001/09/18 16:16:21 kschultz Exp $ */
 
 /*
  * Mesa 3-D graphics library
@@ -161,7 +161,7 @@ _mesa_Lightfv( GLenum light, GLenum pname, const GLfloat *params )
 	 return;
       FLUSH_VERTICES(ctx, _NEW_LIGHT);
       l->SpotCutoff = params[0];
-      l->_CosCutoff = cos(params[0]*DEG2RAD);
+      l->_CosCutoff = (GLfloat) cos(params[0]*DEG2RAD);
       if (l->_CosCutoff < 0)
 	 l->_CosCutoff = 0;
       if (l->SpotCutoff != 180.0F)
@@ -664,7 +664,6 @@ void _mesa_update_material( GLcontext *ctx,
       foreach (light, list) {
 	 SCALE_3V( light->_MatDiffuse[0], light->Diffuse, mat->Diffuse );
       }
-      ctx->Light._BaseAlpha[0] = CLAMP( mat->Diffuse[3], 0.0, 1.0 );
    }
    if (bitmask & BACK_DIFFUSE_BIT) {
       struct gl_material *mat = &ctx->Light.Material[1];
@@ -672,7 +671,6 @@ void _mesa_update_material( GLcontext *ctx,
       foreach (light, list) {
 	 SCALE_3V( light->_MatDiffuse[1], light->Diffuse, mat->Diffuse );
       }
-      ctx->Light._BaseAlpha[1] = CLAMP( mat->Diffuse[3], 0.0, 1.0 );
    }
 
    /* update material specular values */
@@ -801,7 +799,6 @@ void _mesa_update_color_material( GLcontext *ctx,
       foreach (light, list) {
 	 SCALE_3V( light->_MatDiffuse[0], light->Diffuse, mat->Diffuse );
       }
-      ctx->Light._BaseAlpha[0] = CLAMP( mat->Diffuse[3], 0.0, 1.0 );
    }
 
    if (bitmask & BACK_DIFFUSE_BIT) {
@@ -810,7 +807,6 @@ void _mesa_update_color_material( GLcontext *ctx,
       foreach (light, list) {
 	 SCALE_3V( light->_MatDiffuse[1], light->Diffuse, mat->Diffuse );
       }
-      ctx->Light._BaseAlpha[1] = CLAMP( mat->Diffuse[3], 0.0, 1.0 );
    }
 
    /* update light->_MatSpecular = light's specular * material's specular */
@@ -1062,7 +1058,7 @@ static void validate_spot_exp_table( struct gl_light *l )
 	    clamp = 1;
 	 }
       }
-      l->_SpotExpTable[i][0] = tmp;
+      l->_SpotExpTable[i][0] = (GLfloat) tmp;
    }
    for (i = 0; i < EXP_TABLE_SIZE - 1; i++) {
       l->_SpotExpTable[i][1] = (l->_SpotExpTable[i+1][0] -
@@ -1116,7 +1112,7 @@ static void validate_shine_table( GLcontext *ctx, GLuint i, GLfloat shininess )
                x = 0.005;
             t = pow(x, shininess);
 	    if (t > 1e-20)
-	       m[j] = t;
+	       m[j] = (GLfloat) t;
 	    else
 	       m[j] = 0.0;
 	 }
@@ -1216,9 +1212,6 @@ _mesa_update_lighting( GLcontext *ctx )
 	 ACC_SCALE_3V(ctx->Light._BaseColor[side],
 		      ctx->Light.Model.Ambient,
 		      mat->Ambient);
-
-	 ctx->Light._BaseAlpha[side] = 
-	    CLAMP( ctx->Light.Material[side].Diffuse[3], 0.0, 1.0 );
       }
 
       foreach (light, &ctx->Light.EnabledList) {
@@ -1232,7 +1225,7 @@ _mesa_update_lighting( GLcontext *ctx )
       }
    }
    else {
-      static const GLfloat ci[3] = { .30, .59, .11 };
+      static const GLfloat ci[3] = { .30F, .59F, .11F };
       foreach(light, &ctx->Light.EnabledList) {
 	 light->_dli = DOT3(ci, light->Diffuse);
 	 light->_sli = DOT3(ci, light->Specular);
@@ -1307,7 +1300,7 @@ _mesa_compute_light_positions( GLcontext *ctx )
 	       double x = PV_dot_dir * (EXP_TABLE_SIZE-1);
 	       int k = (int) x;
 	       light->_VP_inf_spot_attenuation =
-		  (light->_SpotExpTable[k][0] +
+		  (GLfloat) (light->_SpotExpTable[k][0] +
 		   (x-k)*light->_SpotExpTable[k][1]);
 	    }
 	    else {
