@@ -1,8 +1,8 @@
-/* $Id: mmath.h,v 1.39 2001/06/15 15:22:07 brianp Exp $ */
+/* $Id: mmath.h,v 1.41.2.6 2001/12/05 21:18:59 alanh Exp $ */
 
 /*
  * Mesa 3-D graphics library
- * Version:  3.5
+ * Version:  4.0
  *
  * Copyright (C) 1999-2001  Brian Paul   All Rights Reserved.
  *
@@ -199,8 +199,8 @@ do {						\
 #endif
 
 
-#if defined(__i386__) || defined(__sparc__) || ( defined(__alpha__) && \
-	      ( defined( __IEEE_FLOAT ) || !defined( VMS ) ) )
+#if defined(__i386__) || defined(__sparc__) || defined(__s390x__) || \
+    ( defined(__alpha__) && ( defined( __IEEE_FLOAT ) || !defined( VMS ) ) )
 #define USE_IEEE
 #endif
 
@@ -476,11 +476,11 @@ extern float _mesa_ubyte_to_float_color_tab[256];
 #define FLOAT_TO_INT(X)		( (GLint) (2147483647.0 * (X)) )
 
 
-#define BYTE_TO_UBYTE(b)   ((b) < 0 ? 0 : (GLubyte) (b))
-#define SHORT_TO_UBYTE(s)  ((s) < 0 ? 0 : (GLubyte) ((s) >> 7))
-#define USHORT_TO_UBYTE(s) (GLubyte) ((s) >> 8)
-#define INT_TO_UBYTE(i)    ((i) < 0 ? 0 : (GLubyte) ((i) >> 23))
-#define UINT_TO_UBYTE(i)   (GLubyte) ((i) >> 24)
+#define BYTE_TO_UBYTE(b)   ((GLubyte) ((b) < 0 ? 0 : (GLubyte) (b)))
+#define SHORT_TO_UBYTE(s)  ((GLubyte) ((s) < 0 ? 0 : (GLubyte) ((s) >> 7)))
+#define USHORT_TO_UBYTE(s) ((GLubyte) ((s) >> 8))
+#define INT_TO_UBYTE(i)    ((GLubyte) ((i) < 0 ? 0 : (GLubyte) ((i) >> 23)))
+#define UINT_TO_UBYTE(i)   ((GLubyte) ((i) >> 24))
 
 
 #define BYTE_TO_USHORT(b)  ((b) < 0 ? 0 : ((GLushort) (((b) * 65535) / 255)))
@@ -536,6 +536,21 @@ do {						\
    dst[2] = LINTERP( t, out[2], in[2] );	\
 } while (0)
 
+#define INTERP_4CHAN( t, dst, out, in )		\
+do {						\
+   INTERP_CHAN( t, dst[0], out[0], in[0] );	\
+   INTERP_CHAN( t, dst[1], out[1], in[1] );	\
+   INTERP_CHAN( t, dst[2], out[2], in[2] );	\
+   INTERP_CHAN( t, dst[3], out[3], in[3] );	\
+} while (0)
+
+#define INTERP_3CHAN( t, dst, out, in )		\
+do {						\
+   INTERP_CHAN( t, dst[0], out[0], in[0] );	\
+   INTERP_CHAN( t, dst[1], out[1], in[1] );	\
+   INTERP_CHAN( t, dst[2], out[2], in[2] );	\
+} while (0)
+
 #define INTERP_SZ( t, vec, to, out, in, sz )			\
 do {								\
    switch (sz) {						\
@@ -567,7 +582,28 @@ do {								\
 #define PosFloatToFixed(X)      FloatToFixed(X)
 #define SignedFloatToFixed(X)   FloatToFixed(X)
 
-
+#ifdef USE_IEEE
+/* LITTLE_ENDIAN - add other architectures here */
+#if defined(__i386__) || defined(__alpha__)
+#define IEEE_INF 0x7fc00000 
+#define IEEE_NAN 0x7f800000
+/* BIG_ENDIAN - add other architectures here */
+#elif defined(__mc68000__) || defined(__ppc__) || defined(__sparc__)
+#define IEEE_INF 0x7fffffff
+#define IEEE_NAN 0x7f800000
+#else
+#error "Need to define IEEE_INF and IEEE_NAN for your architecture"
+#endif
+/* Returns TRUE for x == Inf or x == NaN. */
+static INLINE int IS_INF_OR_NAN( float x )
+{
+   union {float f; int i;} tmp;
+   tmp.f = x;
+   return tmp.i == IEEE_INF || tmp.i == IEEE_NAN;
+}
+#else
+#define IS_INF_OR_NAN(x)        (!finite(x)) 
+#endif
 
 extern void
 _mesa_init_math(void);
