@@ -1,4 +1,4 @@
-/* $Id: mtypes.h,v 1.46 2001/06/13 14:56:14 brianp Exp $ */
+/* $Id: mtypes.h,v 1.51.2.1 2001/11/06 15:51:07 brianp Exp $ */
 
 /*
  * Mesa 3-D graphics library
@@ -74,6 +74,8 @@
    typedef GLbyte GLaccum;
 #elif ACCUM_BITS==16
    typedef GLshort GLaccum;
+#elif ACCUM_BITS==32
+   typedef GLfloat GLaccum;
 #else
 #  error "illegal number of accumulation bits"
 #endif
@@ -309,6 +311,7 @@ struct gl_current_attrib {
    GLfloat RasterPos[4];			/* Current raster position */
    GLfloat RasterDistance;			/* Current raster distance */
    GLfloat RasterColor[4];			/* Current raster color */
+   GLfloat RasterSecondaryColor[4];             /* Current rast 2ndary color */
    GLuint  RasterIndex;				/* Current raster index */
    GLfloat *RasterTexCoord;			/* Current raster texcoord*/
    GLfloat RasterMultiTexCoord[MAX_TEXTURE_UNITS][4];
@@ -381,6 +384,7 @@ struct gl_enable_attrib {
    GLboolean SampleAlphaToOne;       /* GL_ARB_multisample */
    GLboolean SampleCoverage;         /* GL_ARB_multisample */
    GLboolean SampleCoverageInvert;   /* GL_ARB_multisample */
+   GLboolean RasterPositionUnclipped; /* GL_IBM_rasterpos_clip */
    GLuint Texture[MAX_TEXTURE_UNITS];
    GLuint TexGen[MAX_TEXTURE_UNITS];
 };
@@ -492,7 +496,6 @@ struct gl_light_attrib {
    GLboolean _NeedVertices;		/* Use fast shader? */
    GLuint  _Flags;		        /* LIGHT_* flags, see below */
    GLfloat _BaseColor[2][3];
-   GLfloat _BaseAlpha[2];
 };
 
 
@@ -951,6 +954,7 @@ struct gl_transform_attrib {
    GLubyte   _AnyClip;				/* How many ClipEnabled? */
    GLboolean Normalize;				/* Normalize all normals? */
    GLboolean RescaleNormals;			/* GL_EXT_rescale_normal */
+   GLboolean RasterPositionUnclipped;           /* GL_IBM_rasterpos_clip */
 };
 
 
@@ -1161,6 +1165,7 @@ struct gl_constants {
    GLint MaxCubeTextureLevels;
    GLuint MaxTextureUnits;
    GLfloat MaxTextureMaxAnisotropy;	/* GL_EXT_texture_filter_anisotropic */
+   GLfloat MaxTextureLodBias;           /* GL_EXT_texture_lod_bias */
    GLuint MaxArrayLockSize;
    GLint SubPixelBits;
    GLfloat MinPointSize, MaxPointSize;		/* aliased */
@@ -1198,6 +1203,7 @@ struct gl_extensions {
    GLboolean ARB_texture_cube_map;
    GLboolean ARB_texture_env_combine;
    GLboolean ARB_texture_env_dot3;
+   GLboolean ARB_texture_mirrored_repeat;
    GLboolean EXT_blend_color;
    GLboolean EXT_blend_func_separate;
    GLboolean EXT_blend_logic_op;
@@ -1226,6 +1232,7 @@ struct gl_extensions {
    GLboolean EXT_texture_lod_bias;
    GLboolean EXT_vertex_array_set;
    GLboolean HP_occlusion_test;
+   GLboolean IBM_rasterpos_clip;
    GLboolean INGR_blend_func_separate;
    GLboolean MESA_window_pos;
    GLboolean MESA_resize_buffers;
