@@ -1,8 +1,8 @@
-/* $Id: s_blend.c,v 1.7 2001/03/19 02:25:36 keithw Exp $ */
+/* $Id: s_blend.c,v 1.9.2.1 2001/12/13 16:14:17 brianp Exp $ */
 
 /*
  * Mesa 3-D graphics library
- * Version:  3.5
+ * Version:  4.0.1
  *
  * Copyright (C) 1999-2001  Brian Paul   All Rights Reserved.
  *
@@ -25,7 +25,6 @@
  */
 
 
-
 #include "glheader.h"
 #include "context.h"
 #include "macros.h"
@@ -46,6 +45,48 @@
 #endif
 
 
+/*
+ * Special case for glBlendFunc(GL_ZERO, GL_ONE)
+ */
+static void _BLENDAPI
+blend_noop( GLcontext *ctx, GLuint n, const GLubyte mask[],
+            GLchan rgba[][4], CONST GLchan dest[][4] )
+{
+   int i;
+   ASSERT(ctx->Color.BlendEquation==GL_FUNC_ADD_EXT);
+   ASSERT(ctx->Color.BlendSrcRGB==GL_ZERO);
+   ASSERT(ctx->Color.BlendDstRGB==GL_ONE);
+   (void) ctx;
+
+   for (i = 0; i < n; i++) {
+      if (mask[i]) {
+         rgba[i][RCOMP] = dest[i][RCOMP];
+         rgba[i][GCOMP] = dest[i][GCOMP];
+         rgba[i][BCOMP] = dest[i][BCOMP];
+         rgba[i][ACOMP] = dest[i][ACOMP];
+      }
+   }
+}
+
+
+/*
+ * Special case for glBlendFunc(GL_ONE, GL_ZERO)
+ */
+static void _BLENDAPI
+blend_replace( GLcontext *ctx, GLuint n, const GLubyte mask[],
+               GLchan rgba[][4], CONST GLchan dest[][4] )
+{
+   ASSERT(ctx->Color.BlendEquation==GL_FUNC_ADD_EXT);
+   ASSERT(ctx->Color.BlendSrcRGB==GL_ONE);
+   ASSERT(ctx->Color.BlendDstRGB==GL_ZERO);
+   (void) ctx;
+   (void) n;
+   (void) mask;
+   (void) rgba;
+   (void) dest;
+}
+
+
 /*
  * Common transparency blending mode.
  */
@@ -216,6 +257,12 @@ blend_modulate( GLcontext *ctx, GLuint n, const GLubyte mask[],
 
    for (i=0;i<n;i++) {
       if (mask[i]) {
+#if CHAN_TYPE == GL_FLOAT
+         rgba[i][RCOMP] = rgba[i][RCOMP] * dest[i][RCOMP];
+         rgba[i][GCOMP] = rgba[i][GCOMP] * dest[i][GCOMP];
+         rgba[i][BCOMP] = rgba[i][BCOMP] * dest[i][BCOMP];
+         rgba[i][ACOMP] = rgba[i][ACOMP] * dest[i][ACOMP];
+#else
          GLint r = (rgba[i][RCOMP] * dest[i][RCOMP]) >> 8;
          GLint g = (rgba[i][GCOMP] * dest[i][GCOMP]) >> 8;
          GLint b = (rgba[i][BCOMP] * dest[i][BCOMP]) >> 8;
@@ -224,6 +271,7 @@ blend_modulate( GLcontext *ctx, GLuint n, const GLubyte mask[],
          rgba[i][GCOMP] = (GLchan) g;
          rgba[i][BCOMP] = (GLchan) b;
          rgba[i][ACOMP] = (GLchan) a;
+#endif
       }
    }
 }
@@ -334,7 +382,7 @@ blend_general( GLcontext *ctx, GLuint n, const GLubyte mask[],
             default:
                /* this should never happen */
                _mesa_problem(ctx, "Bad blend source RGB factor in do_blend");
-	       return;
+               return;
          }
 
          /* Source Alpha factor */
@@ -579,7 +627,8 @@ void _swrast_choose_blend_func( GLcontext *ctx )
    else if (eq==GL_FUNC_ADD_EXT && srcRGB==GL_SRC_ALPHA
 	    && dstRGB==GL_ONE_MINUS_SRC_ALPHA)
    {
-#if defined(USE_MMX_ASM)
+      /* XXX It looks like the MMX blend code is broken.  Disable for now. */
+#if 0 && defined(USE_MMX_ASM)
       if ( cpu_has_mmx ) {
 	 SWRAST_CONTEXT(ctx)->BlendFunc = _mesa_mmx_blend_transparency;
       }
@@ -603,6 +652,12 @@ void _swrast_choose_blend_func( GLcontext *ctx )
    else if (eq==GL_MAX_EXT) {
       SWRAST_CONTEXT(ctx)->BlendFunc = blend_max;
    }
+   else if (eq==GL_FUNC_ADD_EXT && srcRGB == GL_ZERO && dstRGB == GL_ONE) {
+      SWRAST_CONTEXT(ctx)->BlendFunc = blend_noop;
+   }
+   else if (eq==GL_FUNC_ADD_EXT && srcRGB == GL_ONE && dstRGB == GL_ZERO) {
+      SWRAST_CONTEXT(ctx)->BlendFunc = blend_replace;
+   }
    else {
       SWRAST_CONTEXT(ctx)->BlendFunc = blend_general;
    }
