@@ -1,8 +1,8 @@
-/* $Id: texstate.c,v 1.54 2001/06/18 17:26:08 brianp Exp $ */
+/* $Id: texstate.c,v 1.58.2.2 2001/12/13 16:02:32 brianp Exp $ */
 
 /*
  * Mesa 3-D graphics library
- * Version:  3.5
+ * Version:  4.0.1
  *
  * Copyright (C) 1999-2001  Brian Paul   All Rights Reserved.
  *
@@ -68,7 +68,7 @@ _mesa_TexEnvfv( GLenum target, GLenum pname, const GLfloat *param )
    struct gl_texture_unit *texUnit = &ctx->Texture.Unit[ctx->Texture.CurrentUnit];
    ASSERT_OUTSIDE_BEGIN_END(ctx);
 
-#define ERROR(errCode, msg, value)				\
+#define TE_ERROR(errCode, msg, value)				\
    {								\
       char s[100];						\
       sprintf(s, msg, _mesa_lookup_enum_by_nr(value));		\
@@ -83,14 +83,14 @@ _mesa_TexEnvfv( GLenum target, GLenum pname, const GLfloat *param )
 	 switch (mode) {
 	 case GL_ADD:
 	    if (!ctx->Extensions.EXT_texture_env_add) {
-	       ERROR(GL_INVALID_ENUM, "glTexEnv(param=%s)", mode);
+	       TE_ERROR(GL_INVALID_ENUM, "glTexEnv(param=%s)", mode);
 	       return;
 	    }
 	    break;
 	 case GL_COMBINE_EXT:
 	    if (!ctx->Extensions.EXT_texture_env_combine &&
                 !ctx->Extensions.ARB_texture_env_combine) {
-	       ERROR(GL_INVALID_ENUM, "glTexEnv(param=%s)", mode);
+	       TE_ERROR(GL_INVALID_ENUM, "glTexEnv(param=%s)", mode);
 	       return;
 	    }
 	    break;
@@ -100,7 +100,7 @@ _mesa_TexEnvfv( GLenum target, GLenum pname, const GLfloat *param )
 	 case GL_REPLACE:
 	    break;
 	 default:
-            ERROR(GL_INVALID_ENUM, "glTexEnv(param=%s)", mode);
+            TE_ERROR(GL_INVALID_ENUM, "glTexEnv(param=%s)", mode);
 	    return;
 	 }
 
@@ -136,26 +136,20 @@ _mesa_TexEnvfv( GLenum target, GLenum pname, const GLfloat *param )
 	       break;
             case GL_SUBTRACT_ARB:
                if (!ctx->Extensions.ARB_texture_env_combine) {
-                  ERROR(GL_INVALID_ENUM, "glTexEnv(param=%s)", mode);
+                  TE_ERROR(GL_INVALID_ENUM, "glTexEnv(param=%s)", mode);
                   return;
                }
                break;
-	    case GL_DOT3_RGB_EXT:
-	    case GL_DOT3_RGBA_EXT:
-	       if (!ctx->Extensions.EXT_texture_env_dot3) {
-                  ERROR(GL_INVALID_ENUM, "glTexEnv(param=%s)", mode);
-		  return;
-	       }
-	       break;
 	    case GL_DOT3_RGB_ARB:
 	    case GL_DOT3_RGBA_ARB:
-	       if (!ctx->Extensions.ARB_texture_env_dot3) {
-                  ERROR(GL_INVALID_ENUM, "glTexEnv(param=%s)", mode);
+	       if (!ctx->Extensions.EXT_texture_env_dot3 &&
+                   !ctx->Extensions.ARB_texture_env_dot3) {
+                  TE_ERROR(GL_INVALID_ENUM, "glTexEnv(param=%s)", mode);
 		  return;
 	       }
 	       break;
 	    default:
-               ERROR(GL_INVALID_ENUM, "glTexEnv(param=%s)", mode);
+               TE_ERROR(GL_INVALID_ENUM, "glTexEnv(param=%s)", mode);
 	       return;
 	    }
 	    if (texUnit->CombineModeRGB == mode)
@@ -164,7 +158,7 @@ _mesa_TexEnvfv( GLenum target, GLenum pname, const GLfloat *param )
 	    texUnit->CombineModeRGB = mode;
 	 }
 	 else {
-            ERROR(GL_INVALID_ENUM, "glTexEnv(pname=%s)", pname);
+            TE_ERROR(GL_INVALID_ENUM, "glTexEnv(pname=%s)", pname);
 	    return;
 	 }
          break;
@@ -182,12 +176,12 @@ _mesa_TexEnvfv( GLenum target, GLenum pname, const GLfloat *param )
 	       break;
             case GL_SUBTRACT_ARB:
                if (!ctx->Extensions.ARB_texture_env_combine) {
-                  ERROR(GL_INVALID_ENUM, "glTexEnv(param=%s)", mode);
+                  TE_ERROR(GL_INVALID_ENUM, "glTexEnv(param=%s)", mode);
                   return;
                }
                break;
 	    default:
-               ERROR(GL_INVALID_ENUM, "glTexEnv(param=%s)", mode);
+               TE_ERROR(GL_INVALID_ENUM, "glTexEnv(param=%s)", mode);
 	       return;
 	    }
             if (texUnit->CombineModeA == mode)
@@ -196,7 +190,7 @@ _mesa_TexEnvfv( GLenum target, GLenum pname, const GLfloat *param )
             texUnit->CombineModeA = mode;
 	 }
 	 else {
-            ERROR(GL_INVALID_ENUM, "glTexEnv(pname=%s)", pname);
+            TE_ERROR(GL_INVALID_ENUM, "glTexEnv(pname=%s)", pname);
 	    return;
 	 }
 	 break;
@@ -218,12 +212,12 @@ _mesa_TexEnvfv( GLenum target, GLenum pname, const GLfloat *param )
 	       texUnit->CombineSourceRGB[s] = source;
 	       break;
 	    default:
-               ERROR(GL_INVALID_ENUM, "glTexEnv(param=%s)", source);
+               TE_ERROR(GL_INVALID_ENUM, "glTexEnv(param=%s)", source);
 	       return;
 	    }
 	 }
 	 else {
-            ERROR(GL_INVALID_ENUM, "glTexEnv(pname=%s)", pname);
+            TE_ERROR(GL_INVALID_ENUM, "glTexEnv(pname=%s)", pname);
 	    return;
 	 }
 	 break;
@@ -244,12 +238,12 @@ _mesa_TexEnvfv( GLenum target, GLenum pname, const GLfloat *param )
 	       texUnit->CombineSourceA[s] = source;
 	       break;
 	    default:
-               ERROR(GL_INVALID_ENUM, "glTexEnv(param=%s)", source);
+               TE_ERROR(GL_INVALID_ENUM, "glTexEnv(param=%s)", source);
 	       return;
 	    }
 	 }
 	 else {
-            ERROR(GL_INVALID_ENUM, "glTexEnv(pname=%s)", pname);
+            TE_ERROR(GL_INVALID_ENUM, "glTexEnv(pname=%s)", pname);
 	    return;
 	 }
 	 break;
@@ -270,12 +264,12 @@ _mesa_TexEnvfv( GLenum target, GLenum pname, const GLfloat *param )
 	       texUnit->CombineOperandRGB[s] = operand;
 	       break;
 	    default:
-               ERROR(GL_INVALID_ENUM, "glTexEnv(param=%s)", operand);
+               TE_ERROR(GL_INVALID_ENUM, "glTexEnv(param=%s)", operand);
 	       return;
 	    }
 	 }
 	 else {
-            ERROR(GL_INVALID_ENUM, "glTexEnv(pname=%s)", pname);
+            TE_ERROR(GL_INVALID_ENUM, "glTexEnv(pname=%s)", pname);
 	    return;
 	 }
 	 break;
@@ -294,12 +288,12 @@ _mesa_TexEnvfv( GLenum target, GLenum pname, const GLfloat *param )
 	       texUnit->CombineOperandA[pname-GL_OPERAND0_ALPHA_EXT] = operand;
 	       break;
 	    default:
-               ERROR(GL_INVALID_ENUM, "glTexEnv(param=%s)", operand);
+               TE_ERROR(GL_INVALID_ENUM, "glTexEnv(param=%s)", operand);
 	       return;
 	    }
 	 }
 	 else {
-            ERROR(GL_INVALID_ENUM, "glTexEnv(pname=%s)", pname);
+            TE_ERROR(GL_INVALID_ENUM, "glTexEnv(pname=%s)", pname);
 	    return;
 	 }
 	 break;
@@ -317,12 +311,12 @@ _mesa_TexEnvfv( GLenum target, GLenum pname, const GLfloat *param )
 	       FLUSH_VERTICES(ctx, _NEW_TEXTURE);
 	       texUnit->CombineOperandRGB[2] = operand;
 	    default:
-               ERROR(GL_INVALID_ENUM, "glTexEnv(param=%s)", operand);
+               TE_ERROR(GL_INVALID_ENUM, "glTexEnv(param=%s)", operand);
 	       return;
 	    }
 	 }
 	 else {
-            ERROR(GL_INVALID_ENUM, "glTexEnv(pname=%s)", pname);
+            TE_ERROR(GL_INVALID_ENUM, "glTexEnv(pname=%s)", pname);
 	    return;
 	 }
 	 break;
@@ -339,12 +333,12 @@ _mesa_TexEnvfv( GLenum target, GLenum pname, const GLfloat *param )
 	       texUnit->CombineOperandA[2] = operand;
 	       break;
 	    default:
-               ERROR(GL_INVALID_ENUM, "glTexEnv(param=%s)", operand);
+               TE_ERROR(GL_INVALID_ENUM, "glTexEnv(param=%s)", operand);
 	       return;
 	    }
 	 }
 	 else {
-            ERROR(GL_INVALID_ENUM, "glTexEnv(pname=%s)", pname);
+            TE_ERROR(GL_INVALID_ENUM, "glTexEnv(pname=%s)", pname);
 	    return;
 	 }
 	 break;
@@ -372,7 +366,7 @@ _mesa_TexEnvfv( GLenum target, GLenum pname, const GLfloat *param )
 	    texUnit->CombineScaleShiftRGB = newshift;
 	 }
 	 else {
-            ERROR(GL_INVALID_ENUM, "glTexEnv(pname=%s)", pname);
+            TE_ERROR(GL_INVALID_ENUM, "glTexEnv(pname=%s)", pname);
 	    return;
 	 }
 	 break;
@@ -400,7 +394,7 @@ _mesa_TexEnvfv( GLenum target, GLenum pname, const GLfloat *param )
 	    texUnit->CombineScaleShiftA = newshift;
 	 }
 	 else {
-            ERROR(GL_INVALID_ENUM, "glTexEnv(pname=%s)", pname);
+            TE_ERROR(GL_INVALID_ENUM, "glTexEnv(pname=%s)", pname);
 	    return;
 	 }
 	 break;
@@ -419,10 +413,11 @@ _mesa_TexEnvfv( GLenum target, GLenum pname, const GLfloat *param )
 	 if (texUnit->LodBias == param[0])
 	    return;
 	 FLUSH_VERTICES(ctx, _NEW_TEXTURE);
-	 texUnit->LodBias = param[0];
+         texUnit->LodBias = CLAMP(param[0], -ctx->Const.MaxTextureLodBias,
+                                  ctx->Const.MaxTextureLodBias);
 	 break;
       default:
-         ERROR(GL_INVALID_ENUM, "glTexEnv(pname=%s)", pname);
+         TE_ERROR(GL_INVALID_ENUM, "glTexEnv(pname=%s)", pname);
 	 return;
       }
    }
@@ -902,6 +897,7 @@ _mesa_TexParameterfv( GLenum target, GLenum pname, const GLfloat *params )
              || eparam==GL_LINEAR_MIPMAP_NEAREST
              || eparam==GL_NEAREST_MIPMAP_LINEAR
              || eparam==GL_LINEAR_MIPMAP_LINEAR) {
+            FLUSH_VERTICES(ctx, _NEW_TEXTURE);
             texObj->MinFilter = eparam;
          }
          else {
@@ -915,6 +911,7 @@ _mesa_TexParameterfv( GLenum target, GLenum pname, const GLfloat *params )
             return;
 
          if (eparam==GL_NEAREST || eparam==GL_LINEAR) {
+            FLUSH_VERTICES(ctx, _NEW_TEXTURE);
             texObj->MagFilter = eparam;
          }
          else {
@@ -929,7 +926,10 @@ _mesa_TexParameterfv( GLenum target, GLenum pname, const GLfloat *params )
              eparam==GL_REPEAT ||
              eparam==GL_CLAMP_TO_EDGE ||
              (eparam == GL_CLAMP_TO_BORDER_ARB &&
-              ctx->Extensions.ARB_texture_border_clamp)) {
+              ctx->Extensions.ARB_texture_border_clamp) ||
+             (eparam == GL_MIRRORED_REPEAT_ARB &&
+              ctx->Extensions.ARB_texture_mirrored_repeat)) {
+            FLUSH_VERTICES(ctx, _NEW_TEXTURE);
             texObj->WrapS = eparam;
          }
          else {
@@ -944,7 +944,10 @@ _mesa_TexParameterfv( GLenum target, GLenum pname, const GLfloat *params )
              eparam==GL_REPEAT ||
              eparam==GL_CLAMP_TO_EDGE ||
              (eparam == GL_CLAMP_TO_BORDER_ARB &&
-              ctx->Extensions.ARB_texture_border_clamp)) {
+              ctx->Extensions.ARB_texture_border_clamp) ||
+             (eparam == GL_MIRRORED_REPEAT_ARB &&
+              ctx->Extensions.ARB_texture_mirrored_repeat)) {
+            FLUSH_VERTICES(ctx, _NEW_TEXTURE);
             texObj->WrapT = eparam;
          }
          else {
@@ -959,7 +962,10 @@ _mesa_TexParameterfv( GLenum target, GLenum pname, const GLfloat *params )
              eparam==GL_REPEAT ||
              eparam==GL_CLAMP_TO_EDGE ||
              (eparam == GL_CLAMP_TO_BORDER_ARB &&
-              ctx->Extensions.ARB_texture_border_clamp)) {
+              ctx->Extensions.ARB_texture_border_clamp) ||
+             (eparam == GL_MIRRORED_REPEAT_ARB &&
+              ctx->Extensions.ARB_texture_mirrored_repeat)) {
+            FLUSH_VERTICES(ctx, _NEW_TEXTURE);
             texObj->WrapR = eparam;
          }
          else {
@@ -967,15 +973,22 @@ _mesa_TexParameterfv( GLenum target, GLenum pname, const GLfloat *params )
          }
          break;
       case GL_TEXTURE_BORDER_COLOR:
+         FLUSH_VERTICES(ctx, _NEW_TEXTURE);
          UNCLAMPED_FLOAT_TO_CHAN(texObj->BorderColor[0], params[0]);
          UNCLAMPED_FLOAT_TO_CHAN(texObj->BorderColor[1], params[1]);
          UNCLAMPED_FLOAT_TO_CHAN(texObj->BorderColor[2], params[2]);
          UNCLAMPED_FLOAT_TO_CHAN(texObj->BorderColor[3], params[3]);
          break;
       case GL_TEXTURE_MIN_LOD:
+         if (texObj->MinLod == params[0])
+            return;
+         FLUSH_VERTICES(ctx, _NEW_TEXTURE);
          texObj->MinLod = params[0];
          break;
       case GL_TEXTURE_MAX_LOD:
+         if (texObj->MaxLod == params[0])
+            return;
+         FLUSH_VERTICES(ctx, _NEW_TEXTURE);
          texObj->MaxLod = params[0];
          break;
       case GL_TEXTURE_BASE_LEVEL:
@@ -983,6 +996,7 @@ _mesa_TexParameterfv( GLenum target, GLenum pname, const GLfloat *params )
             _mesa_error(ctx, GL_INVALID_VALUE, "glTexParameter(param)" );
             return;
          }
+         FLUSH_VERTICES(ctx, _NEW_TEXTURE);
          texObj->BaseLevel = (GLint) params[0];
          break;
       case GL_TEXTURE_MAX_LEVEL:
@@ -990,10 +1004,12 @@ _mesa_TexParameterfv( GLenum target, GLenum pname, const GLfloat *params )
             _mesa_error(ctx, GL_INVALID_VALUE, "glTexParameter(param)" );
             return;
          }
+         FLUSH_VERTICES(ctx, _NEW_TEXTURE);
          texObj->MaxLevel = (GLint) params[0];
          break;
       case GL_TEXTURE_PRIORITY:
          /* (keithh@netcomuk.co.uk) */
+         FLUSH_VERTICES(ctx, _NEW_TEXTURE);
          texObj->Priority = CLAMP( params[0], 0.0F, 1.0F );
          break;
       case GL_TEXTURE_MAX_ANISOTROPY_EXT:
@@ -1002,19 +1018,23 @@ _mesa_TexParameterfv( GLenum target, GLenum pname, const GLfloat *params )
 	       _mesa_error(ctx, GL_INVALID_VALUE, "glTexParameter(param)" );
 	       return;
 	    }
+            FLUSH_VERTICES(ctx, _NEW_TEXTURE);
             texObj->MaxAnisotropy = params[0];
          }
          else {
-            _mesa_error(ctx, GL_INVALID_ENUM, "glTexParameter(pname)");
+            _mesa_error(ctx, GL_INVALID_ENUM,
+                        "glTexParameter(pname=GL_MAX_TEXTURE_ANISOTROPY_EXT)");
             return;
          }
          break;
       case GL_TEXTURE_COMPARE_SGIX:
          if (ctx->Extensions.SGIX_shadow) {
+            FLUSH_VERTICES(ctx, _NEW_TEXTURE);
             texObj->CompareFlag = params[0] ? GL_TRUE : GL_FALSE;
          }
          else {
-            _mesa_error(ctx, GL_INVALID_ENUM, "glTexParameter(pname)");
+            _mesa_error(ctx, GL_INVALID_ENUM,
+                        "glTexParameter(pname=GL_TEXTURE_COMPARE_SGIX)");
             return;
          }
          break;
@@ -1023,6 +1043,7 @@ _mesa_TexParameterfv( GLenum target, GLenum pname, const GLfloat *params )
             GLenum op = (GLenum) params[0];
             if (op == GL_TEXTURE_LEQUAL_R_SGIX ||
                 op == GL_TEXTURE_GEQUAL_R_SGIX) {
+               FLUSH_VERTICES(ctx, _NEW_TEXTURE);
                texObj->CompareOperator = op;
             }
             else {
@@ -1030,34 +1051,42 @@ _mesa_TexParameterfv( GLenum target, GLenum pname, const GLfloat *params )
             }
          }
          else {
-            _mesa_error(ctx, GL_INVALID_ENUM, "glTexParameter(pname)");
+            _mesa_error(ctx, GL_INVALID_ENUM,
+                    "glTexParameter(pname=GL_TEXTURE_COMPARE_OPERATOR_SGIX)");
             return;
          }
          break;
       case GL_SHADOW_AMBIENT_SGIX:
          if (ctx->Extensions.SGIX_shadow_ambient) {
+            FLUSH_VERTICES(ctx, _NEW_TEXTURE);
             UNCLAMPED_FLOAT_TO_CHAN(texObj->ShadowAmbient, params[0]);
          }
          else {
-            _mesa_error(ctx, GL_INVALID_ENUM, "glTexParameter(pname)");
+            _mesa_error(ctx, GL_INVALID_ENUM,
+                        "glTexParameter(pname=GL_SHADOW_AMBIENT_SGIX)");
             return;
          }
          break;
       case GL_GENERATE_MIPMAP_SGIS:
          if (ctx->Extensions.SGIS_generate_mipmap) {
+            FLUSH_VERTICES(ctx, _NEW_TEXTURE);
             texObj->GenerateMipmap = params[0] ? GL_TRUE : GL_FALSE;
          }
          else {
-            _mesa_error(ctx, GL_INVALID_ENUM, "glTexParameter(pname)");
+            _mesa_error(ctx, GL_INVALID_ENUM,
+                        "glTexParameter(pname=GL_GENERATE_MIPMAP_SGIS)");
             return;
          }
          break;
       default:
-         _mesa_error( ctx, GL_INVALID_ENUM, "glTexParameter(pname)" );
+         {
+            char s[100];
+            sprintf(s, "glTexParameter(pname=0x%x)", pname);
+            _mesa_error( ctx, GL_INVALID_ENUM, s);
+         }
          return;
    }
 
-   ctx->NewState |= _NEW_TEXTURE;
    texObj->Complete = GL_FALSE;
 
    if (ctx->Driver.TexParameter) {
@@ -1724,10 +1753,10 @@ void
 _mesa_TexGeniv(GLenum coord, GLenum pname, const GLint *params )
 {
    GLfloat p[4];
-   p[0] = params[0];
-   p[1] = params[1];
-   p[2] = params[2];
-   p[3] = params[3];
+   p[0] = (GLfloat) params[0];
+   p[1] = (GLfloat) params[1];
+   p[2] = (GLfloat) params[2];
+   p[3] = (GLfloat) params[3];
    _mesa_TexGenfv(coord, pname, p);
 }
 
@@ -1744,10 +1773,10 @@ void
 _mesa_TexGendv(GLenum coord, GLenum pname, const GLdouble *params )
 {
    GLfloat p[4];
-   p[0] = params[0];
-   p[1] = params[1];
-   p[2] = params[2];
-   p[3] = params[3];
+   p[0] = (GLfloat) params[0];
+   p[1] = (GLfloat) params[1];
+   p[2] = (GLfloat) params[2];
+   p[3] = (GLfloat) params[3];
    _mesa_TexGenfv( coord, pname, p );
 }
 
