@@ -1,4 +1,4 @@
-/* $Id: t_context.c,v 1.19 2001/06/04 16:09:28 keithw Exp $ */
+/* $Id: t_context.c,v 1.22 2001/07/19 15:54:35 brianp Exp $ */
 
 /*
  * Mesa 3-D graphics library
@@ -104,6 +104,7 @@ _tnl_CreateContext( GLcontext *ctx )
 
    tnl->NeedProjCoords = GL_TRUE;
    tnl->LoopbackDListCassettes = GL_FALSE;
+   tnl->CalcDListNormalLengths = GL_TRUE;
 
    /* Hook our functions into exec and compile dispatch tables.
     */
@@ -122,8 +123,8 @@ _tnl_CreateContext( GLcontext *ctx )
    ctx->Driver.CurrentExecPrimitive = PRIM_OUTSIDE_BEGIN_END;
    ctx->Driver.CurrentSavePrimitive = PRIM_UNKNOWN;
 
-   tnl->Driver.RenderTabElts = _tnl_render_tab_elts;
-   tnl->Driver.RenderTabVerts = _tnl_render_tab_verts;
+   tnl->Driver.Render.PrimTabElts = _tnl_render_tab_elts;
+   tnl->Driver.Render.PrimTabVerts = _tnl_render_tab_verts;
 
    
    return GL_TRUE;
@@ -138,6 +139,7 @@ _tnl_DestroyContext( GLcontext *ctx )
    _tnl_array_destroy( ctx );
    _tnl_imm_destroy( ctx );
    _tnl_destroy_pipeline( ctx );
+   _tnl_free_immediate( tnl->freed_immediate );
 
    FREE(tnl);
    ctx->swtnl_context = 0;
@@ -222,3 +224,12 @@ _tnl_need_dlist_loopback( GLcontext *ctx, GLboolean mode )
       tnl->LoopbackDListCassettes = mode;
    }
 }
+
+void
+_tnl_need_dlist_norm_lengths( GLcontext *ctx, GLboolean mode )
+{
+   TNLcontext *tnl = TNL_CONTEXT(ctx);
+   if (tnl->CalcDListNormalLengths != mode) {
+      tnl->CalcDListNormalLengths = mode;
+   }
+}
