@@ -1,4 +1,4 @@
-/* $Id: t_imm_dlist.c,v 1.20 2001/06/04 16:09:28 keithw Exp $ */
+/* $Id: t_imm_dlist.c,v 1.29.2.4 2001/12/13 10:57:55 keithw Exp $ */
 
 /*
  * Mesa 3-D graphics library
@@ -66,6 +66,57 @@ static void execute_compiled_cassette( GLcontext *ctx, void *data );
 static void loopback_compiled_cassette( GLcontext *ctx, struct immediate *IM );
 
 
+static void build_normal_lengths( struct immediate *IM )
+{
+   GLuint i;
+   GLfloat len;
+   GLfloat (*data)[3] = IM->Normal + IM->Start;
+   GLfloat *dest = IM->NormalLengthPtr;
+   GLuint *flags = IM->Flag + IM->Start;
+   GLuint count = IM->Count - IM->Start;
+
+   if (!dest) {
+      dest = IM->NormalLengthPtr = ALIGN_MALLOC( IMM_SIZE*sizeof(GLfloat), 32 );
+      if (!dest) return;
+   }
+   dest += IM->Start;
+
+   len = (GLfloat) LEN_3FV( data[0] );
+   if (len > 0.0F) len = 1.0F / len;
+   
+   for (i = 0 ; i < count ; ) {
+      dest[i] = len;
+      if (flags[++i] & VERT_NORM) {
+	 len = (GLfloat) LEN_3FV( data[i] );
+	 if (len > 0.0F) len = 1.0F / len;
+      }
+   } 
+}
+
+static void fixup_normal_lengths( struct immediate *IM ) 
+{
+   GLuint i;
+   GLfloat len = 1.0F;  /* just to silence warnings */
+   GLfloat (*data)[3] = IM->Normal;
+   GLfloat *dest = IM->NormalLengthPtr;
+   GLuint *flags = IM->Flag;
+
+   for (i = IM->CopyStart ; i <= IM->Start ; i++) {
+      len = (GLfloat) LEN_3FV( data[i] );
+      if (len > 0.0F) len = 1.0F / len;
+      dest[i] = len;
+   } 
+
+   if (i < IM->Count) {
+      while (!(flags[i] & (VERT_NORM|VERT_END_VB))) {
+	 dest[i] = len;
+	 i++;
+      }
+   }
+}
+
+
+
 /* Insert the active immediate struct onto the display list currently
  * being built.
  */
@@ -84,14 +135,14 @@ _tnl_compile_cassette( GLcontext *ctx, struct immediate *IM )
 
    _tnl_compute_orflag( IM, IM->Start );
 
-   /* Need to clear this flag, or fixup gets confused.  (The elements
-    * have been translated away by now.)  
+   /* Need to clear this flag, or fixup gets confused.  (The
+    * array-elements have been translated away by now, so it's ok to
+    * remove it.)
     */
    IM->OrFlag &= ~VERT_ELT;	
    IM->AndFlag &= ~VERT_ELT;	
 
    _tnl_fixup_input( ctx, IM );
-/*     _tnl_print_cassette( IM ); */
 
    node = (TNLvertexcassette *)
       _mesa_alloc_instruction(ctx,
@@ -113,6 +164,10 @@ _tnl_compile_cassette( GLcontext *ctx, struct immediate *IM )
    node->LastMaterial = im->LastMaterial;
    node->MaterialOrMask = im->MaterialOrMask;
    node->MaterialAndMask = im->MaterialAndMask;
+   
+   if (tnl->CalcDListNormalLengths) {
+      build_normal_lengths( im );
+   }
 
    if (ctx->ExecuteFlag) {
       execute_compiled_cassette( ctx, (void *)node );
@@ -174,7 +229,7 @@ static void fixup_compiled_primitives( GLcontext *ctx, struct immediate *IM )
        * set copystart because it might skip materials?
        */
       ASSERT(IM->Start == IM->CopyStart);
-      if (i > IM->CopyStart) {
+      if (i > IM->CopyStart || !(IM->Flag[IM->Start] & VERT_BEGIN)) {
 	 IM->Primitive[IM->CopyStart] = GL_POLYGON+1;
 	 IM->PrimitiveLength[IM->CopyStart] = i - IM->CopyStart;
 	 if (IM->Flag[i] & VERT_END_VB) {
@@ -238,7 +293,7 @@ execute_compiled_cassette( GLcontext *ctx, void *data )
    TNLvertexcassette *node = (TNLvertexcassette *)data;
    struct immediate *IM = node->IM;
 
-/*     fprintf(stderr, "%s\n", __FUNCTION__); */
+/*     fprintf(stderr, "%s IM: %d\n", __FUNCTION__, IM->id);  */
 
    IM->Start = node->Start;
    IM->CopyStart = node->Start;
@@ -254,6 +309,7 @@ execute_compiled_cassette( GLcontext *ctx, void *data )
    IM->MaterialOrMask = node->MaterialOrMask;
    IM->MaterialAndMask = node->MaterialAndMask;
 
+
    if ((MESA_VERBOSE & VERBOSE_DISPLAY_LIST) &&
        (MESA_VERBOSE & VERBOSE_IMMEDIATE))
       _tnl_print_cassette( IM );
@@ -288,9 +344,6 @@ execute_compiled_cassette( GLcontext *ctx, void *data )
       loopback_compiled_cassette( ctx, IM );
       restore_compiled_primitives( ctx, IM );
    }
-   else if (IM->Count == IM->Start) {
-      _tnl_copy_to_current( ctx, IM, IM->OrFlag );
-   }
    else {
       if (ctx->NewState)
 	 _mesa_update_state(ctx);
@@ -303,12 +356,24 @@ execute_compiled_cassette( GLcontext *ctx, void *data )
 
       if (IM->Primitive[IM->LastPrimitive] & PRIM_END)
 	 ctx->Driver.CurrentExecPrimitive = GL_POLYGON+1;
-      else
+      else if ((IM->Primitive[IM->LastPrimitive] & PRIM_BEGIN) ||
+	       (IM->Primitive[IM->LastPrimitive] & PRIM_MODE_MASK) == 
+	       PRIM_OUTSIDE_BEGIN_END) {
 	 ctx->Driver.CurrentExecPrimitive =
 	    IM->Primitive[IM->LastPrimitive] & PRIM_MODE_MASK;
+      }
 
       _tnl_get_exec_copy_verts( ctx, IM );
-      _tnl_run_cassette( ctx, IM );
+
+      if (IM->NormalLengthPtr) 
+	 fixup_normal_lengths( IM );
+      
+      if (IM->Count == IM->Start) 
+	 _tnl_copy_to_current( ctx, IM, IM->OrFlag, IM->LastData );
+      else {
+/*  	 _tnl_print_cassette( IM ); */
+	 _tnl_run_cassette( ctx, IM );
+      }
 
       restore_compiled_primitives( ctx, IM );
    }
@@ -337,6 +402,7 @@ print_compiled_cassette( GLcontext *ctx, void *data )
 	   node->IM->id, node->Start, node->Count);
 
    IM->Start = node->Start;
+   IM->CopyStart = node->Start;
    IM->Count = node->Count;
    IM->BeginState = node->BeginState;
    IM->OrFlag = node->OrFlag;
@@ -356,15 +422,22 @@ _tnl_BeginCallList( GLcontext *ctx, GLuint list )
 {
    (void) ctx;
    (void) list;
-   FLUSH_CURRENT(ctx, 0);
+   FLUSH_CURRENT(ctx, 0);	/* Current immediate is emptied on CallList */
 }
 
 
-/* Called at the tail of a CallList.  Nothing to do.
+/* Called at the tail of a CallList.  Make current immediate aware of
+ * any new to-be-copied vertices.
  */
 void
 _tnl_EndCallList( GLcontext *ctx )
 {
+   GLuint beginstate = 0;
+
+   if (ctx->Driver.CurrentExecPrimitive != PRIM_OUTSIDE_BEGIN_END)
+      beginstate = VERT_BEGIN_0|VERT_BEGIN_1;
+
+   _tnl_reset_exec_input( ctx, TNL_CURRENT_IM(ctx)->Start, beginstate, 0 );
 }
 
 
@@ -493,15 +566,15 @@ static void loopback_compiled_cassette( GLcontext *ctx, struct immediate *IM )
    GLuint *flags = IM->Flag;
    GLuint orflag = IM->OrFlag;
    GLuint j;
-   void (*vertex)( const GLfloat * );
-   void (*texcoordfv[MAX_TEXTURE_UNITS])( GLuint, const GLfloat * );
+   void (GLAPIENTRY *vertex)( const GLfloat * );
+   void (GLAPIENTRY *texcoordfv[MAX_TEXTURE_UNITS])( GLenum, const GLfloat * );
    GLuint maxtex = 0;
    GLuint p, length, prim = 0;
    
    if (orflag & VERT_OBJ_234)
-      vertex = glVertex4fv;
+      vertex = (void (GLAPIENTRY *)(const GLfloat *)) glVertex4fv;
    else
-      vertex = glVertex3fv;
+      vertex = (void (GLAPIENTRY *)(const GLfloat *)) glVertex3fv;
    
    if (orflag & VERT_TEX_ANY) {
       for (j = 0 ; j < ctx->Const.MaxTextureUnits ; j++) {
@@ -525,7 +598,6 @@ static void loopback_compiled_cassette( GLcontext *ctx, struct immediate *IM )
       ASSERT((prim & PRIM_MODE_MASK) <= GL_POLYGON+1);
 
       if (prim & PRIM_BEGIN) {
-/*  	 fprintf(stderr, "begin %s\n", _mesa_prim_name[prim&PRIM_MODE_MASK]); */
 	 glBegin(prim & PRIM_MODE_MASK);
       }
 
@@ -540,14 +612,10 @@ static void loopback_compiled_cassette( GLcontext *ctx, struct immediate *IM )
 	 }
 
 	 if (flags[i] & VERT_NORM) {
-/*  	       fprintf(stderr, "normal %d: %f %f %f\n", i, */
-/*  		       IM->Normal[i][0], IM->Normal[i][1], IM->Normal[i][2]);  */
 	    glNormal3fv(IM->Normal[i]);
 	 }
 
 	 if (flags[i] & VERT_RGBA) {
-/*  	       fprintf(stderr, "color %d: %f %f %f\n", i, */
-/*  		       IM->Color[i][0], IM->Color[i][1], IM->Color[i][2]);  */
 	    glColor4fv( IM->Color[i] );
 	 }
 
@@ -567,22 +635,19 @@ static void loopback_compiled_cassette( GLcontext *ctx, struct immediate *IM )
 	    emit_material( IM->Material[i], IM->MaterialMask[i] );
 
 	 if (flags[i]&VERT_OBJ_234) {
-/*  	       fprintf(stderr, "vertex %d: %f %f %f\n", i, */
-/*  		       IM->Obj[i][0], IM->Obj[i][1], IM->Obj[i][2]); */
 	    vertex( IM->Obj[i] );
 	 }
 	 else if (flags[i] & VERT_EVAL_C1)
-	    glEvalCoord1f(IM->Obj[i][0]);
+	    glEvalCoord1f( IM->Obj[i][0] );
 	 else if (flags[i] & VERT_EVAL_P1)
-	    glEvalPoint1(IM->Obj[i][0]);
+	    glEvalPoint1( (GLint) IM->Obj[i][0] );
 	 else if (flags[i] & VERT_EVAL_C2)
-	    glEvalCoord2f( IM->Obj[i][0], IM->Obj[i][1]);
+	    glEvalCoord2f( IM->Obj[i][0], IM->Obj[i][1] );
 	 else if (flags[i] & VERT_EVAL_P2)
-	    glEvalPoint2( IM->Obj[i][0], IM->Obj[i][1]);
+	    glEvalPoint2( (GLint) IM->Obj[i][0], (GLint) IM->Obj[i][1] );
       }
 
       if (prim & PRIM_END) {
-/*  	 fprintf(stderr, "end\n"); */
 	 glEnd();
       }
    }
